/* Copyright (C) 2009, Chuck Shotton, http://www.shotton.com/
 * Licensed under the Creative Commons Attribution-Share Alike 3.0 license.
 * See http://creativecommons.org/licenses/by-sa/3.0/ for details.
 */

package com.concluent.rssNimbus;

import javax.jdo.PersistenceManager;
import javax.jdo.annotations.IdentityType;
import javax.jdo.annotations.PersistenceCapable;
import javax.jdo.annotations.Persistent;
import javax.jdo.annotations.PrimaryKey;

/**
 * defines a Subscription object for JDO operations
 * @author Chuck Shotton
 *
 */
@PersistenceCapable(identityType = IdentityType.APPLICATION, detachable="true")
public class SubscriptionObject {

	@PrimaryKey
	@Persistent
	private String key=""; //generated by generateKey, of the form ip_address@url_hash
	@Persistent
	private long timestamp; //last time this subscription was successfully serviced in millis since epoch
	@Persistent
	private String address; //ip address or domain name of subscriber
	@Persistent
	private String protocol;
	@Persistent
	private String notificationProcedure; //probably unused since we only make REST calls
	@Persistent
	private String port;
	@Persistent
	private String path;
	@Persistent
	private String url;
	@Persistent
	private boolean needsNotification; //probably unused
	@Persistent
	private int failures;
  
	/**
	 * create a unique key for the SubscriptionObject
	 * @param address IP address or domain name of the subscriber
	 * @param url URL of the source publication
	 * @return a unique key of the form address@hashed_url
	 */
	public static String generateKey (String address, String url) {
		return address + "@" + Utility.makeMD5(url);
	}
	
	/**
	 * find a SubscriptionObject by its unique key
	 * @param key a key string of the form created by generateKey
	 * @return SubscriptionObject with the specified key
	 */
	public static SubscriptionObject getSubscriptionByKey (String key) {
		SubscriptionObject osub = null;
		PersistenceManager pm = RssNimbusServlet.PMF.getPersistenceManager ();
		try {
			osub = pm.getObjectById(SubscriptionObject.class, key);
			osub = pm.detachCopy (osub);
		} catch (Exception e) {
			//e.printStackTrace();
		} finally {
			pm.close();
		}
		return osub;
	}
	
	/**
	 * delete the specified SubscriptionObject from the repository
	 * @param s SubscriptionObject to delete
	 * @return success flag
	 */
	public static boolean deleteSubscription (SubscriptionObject s) {
		boolean success = true;
		PersistenceManager pm = RssNimbusServlet.PMF.getPersistenceManager ();
		try {
			SubscriptionObject osub = pm.getObjectById(SubscriptionObject.class, s.getKey());
			pm.deletePersistent(osub);
		} catch (Exception e) {
			e.printStackTrace();
			success = false;
		} finally {
			pm.close();
		}
		return success;
	}

	public SubscriptionObject () {
	}
	
	public SubscriptionObject (String protocol, String address, String port, String notificationProcedure, String path, String url) {
		this.key = generateKey (address, url);
		this.protocol = protocol;
		this.address = address;
		this.port = port;
		this.notificationProcedure = notificationProcedure;
		this.path = path;
		this.url = url;
		this.needsNotification = false;
		this.failures = 0;
		makeTimestamp ();
	}
	
	public boolean saveSubscription () {
		boolean success = true;
		PersistenceManager pm = RssNimbusServlet.PMF.getPersistenceManager ();
		
		try {
			pm.makePersistent(this);
		} catch (Exception e) {
			e.printStackTrace ();
			success = false;
		} finally {
			pm.close ();
		}
		return success;
	}
	
	public String getKey () {
		return this.key;
	}

	public String getURL () {
		return this.url;
	}
	
	public String getProtocol () {
		return this.protocol;
	}
	
	public String getAddress () {
		return this.address;
	}
	
	public String getPort () {
		return this.port;
	}

	public String getPath () {
		return this.path;
	}
	
	public boolean getNeedsNotification () {
		return this.needsNotification;
	}
	
	public int getFailures () {
		return this.failures;
	}
	
	public void setFailures (int fail) {
		this.failures = fail;
	}
	
	public void setNeedsNotification (boolean b) {
		if (b == true);
		this.needsNotification = b;
	}
	
	public long getTimestamp () {
		return this.timestamp;
	}
	
	public void makeTimestamp () {
		this.timestamp = java.lang.System.currentTimeMillis();
	}
	
	public void setTimestamp (long timestamp) {
		this.timestamp = timestamp;
	}

}
