/* Copyright (C) 2009, Chuck Shotton, http://www.shotton.com/
 * Licensed under the Creative Commons Attribution-Share Alike 3.0 license.
 * See http://creativecommons.org/licenses/by-sa/3.0/ for details.
 */

package com.concluent.rssNimbus;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.security.*;
import java.util.*;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;
import com.google.appengine.api.labs.taskqueue.Queue;
import com.google.appengine.api.labs.taskqueue.QueueFactory;
import com.google.appengine.api.labs.taskqueue.TaskOptions;

/**
 * Functions for managing publications and subscriptions used by several restlet handlers
 * @author Chuck Shotton
 *
 */
public class Utility {
	
	private static final int MAX_NOTIFY_ATTEMPTS = 1; //one strike and yer out!
	public static final long SUBSCRIPTION_EXPIRE_TIME = 86400000; //24 hours in millis
	public static final int CONNECT_TIMEOUT = 1500; //wait 1.5 secs to connect (likely ignored by GAE unfortunately)
	public static final int READ_TIMEOUT = 1000;	// wait 1 sec to receive response (also likely ignored)
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * pre-flights the notification handler of one subscriber and confirms a 3rd party subscribe request
	 * @param sub the JDO SubscriptionObject we are to pre-flight
	 * @param challenge the challenge string that must match for us to approve the subscription
	 * @return success or failure
	 */
	public static boolean challengeOneSubscriber (SubscriptionObject sub, String challenge) {
		String res = "";
		boolean ok = true;
		
		try {
			String url = "http://" + sub.getAddress() + ":" + sub.getPort() + sub.getPath();
			
			if (challenge!=null && challenge.length()>0) {
				url += "?url=" + sub.getURL() + "&challenge=" + challenge;
			}
			
System.out.println ("Challenging " + url + " about " + sub.getURL());
			URL u = new URL (url);
			String line = "";
			
			URLConnection uc = u.openConnection ();
			HttpURLConnection huc = (HttpURLConnection) uc;
		
			huc.setConnectTimeout(CONNECT_TIMEOUT);
			huc.setReadTimeout(READ_TIMEOUT);
			huc.setRequestMethod("GET");
			
			BufferedReader in = new BufferedReader (new InputStreamReader (uc.getInputStream ()));
			int rcode = huc.getResponseCode ();
			String response = huc.getResponseMessage ();
			
			while ((line = in.readLine()) != null) {
				res += line + "\n";
			}
System.out.println ("GET response was: " + rcode + ", " + res);			
			in.close ();

/*			BufferedReader reader = new BufferedReader (new InputStreamReader (u.openStream()));
			
			while ((line = reader.readLine()) != null) {
				res += line + "\n";
			}
*/			
			//TODO if we're challenging, confirm the challenge string in the response
			
		} catch (Exception e) {
			ok = false;
			res += "\nError trying to challenge subscriber.";
			e.printStackTrace();
		}
System.out.println ("  result="+res);
		return ok;
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * notify a single subscriber of an update to its subscription
	 * @param sub the JDO SubscriptionObject to be notified
	 * @return success or failure
	 */
	public static boolean notifyOneSubscriber (SubscriptionObject sub) {
		String res = "";
		boolean ok = true;
		PersistenceManager pm = RssNimbusServlet.PMF.getPersistenceManager ();
		
		try {
			String url = "http://" + sub.getAddress() + ":" + sub.getPort() + sub.getPath();
			String postArgs="url=" + sub.getURL();
			System.out.println ("Notifying " + url + " about " + sub.getURL());
			URL u = new URL (url);
			String line = "";

			URLConnection uc = u.openConnection ();
			HttpURLConnection huc = (HttpURLConnection) uc;
		
			huc.setConnectTimeout(CONNECT_TIMEOUT);
			huc.setReadTimeout(READ_TIMEOUT);
			huc.setRequestMethod("POST");
			huc.setDoOutput (true);
			
			OutputStream os = new BufferedOutputStream (uc.getOutputStream ());
			OutputStreamWriter out = new OutputStreamWriter (os, "8859_1");
			
			out.write (postArgs);
			out.flush ();
			out.close ();
			
			BufferedReader in = new BufferedReader (new InputStreamReader (uc.getInputStream ()));
			int rcode = huc.getResponseCode ();
			String response = huc.getResponseMessage ();
			
			while ((line = in.readLine()) != null) {
				res += line + "\n";
			}
System.out.println ("POST response was: " + rcode + ", " + res);			
			in.close ();
		} catch (Exception e) {
			ok = false;
			res += "\nError trying to notify subscriber.";
			e.printStackTrace();
		}
		if (!ok) {
			sub.setFailures(sub.getFailures()+1);
			if (sub.getFailures() > MAX_NOTIFY_ATTEMPTS) {
				System.out.println ("Expiring subscription for " + sub.getAddress() + " url=" + sub.getURL() + " after failures.");
				pm.deletePersistent(sub);
			}
			else {
				sub.saveSubscription();
			}
		}
		else {
			if (sub.getFailures()>0) {
				//notified the host, so reset the previous failures
				sub.setFailures(0);
				sub.saveSubscription();
			}
		}
System.out.println ("  result="+res);
		return ok;
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * notify a single subscriber given the subscriber's unique key
	 * @param key String returned from generateKey
	 * @return success flag
	 */
	public static boolean notifyOneSubscriberByKey (String key) {
		PersistenceManager pm = RssNimbusServlet.PMF.getPersistenceManager ();
		SubscriptionObject sub = null;
		boolean res=false;
		
		try {
			sub = pm.getObjectById(SubscriptionObject.class, key);
			res = Utility.notifyOneSubscriber (sub);
		} catch (Exception e) {
			//e.printStackTrace();
		} finally {
			pm.close();
		}
		
		return res;
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * notify all of the subscribers to a particular publication that it has been updated
	 * @param url the address of the publication that has been updated
	 * @return number of subscribers that will be notified
	 */
	public static int notifyAllSubscribers (String url, boolean queued) {
		int count = 0;
		PersistenceManager pm = RssNimbusServlet.PMF.getPersistenceManager ();
		Query query = pm.newQuery(SubscriptionObject.class);
		query.setFilter("url == urlParam");
		query.declareParameters("String urlParam");
		Queue queue = QueueFactory.getDefaultQueue();
		
		try {
			List<SubscriptionObject> subs = (List <SubscriptionObject>) query.execute (url);
			
			if (subs.iterator().hasNext()) {
				for (SubscriptionObject sub : subs) {
					boolean notified = false;
					if (queued) {
System.out.println ("queuing task for " + sub.getKey());
						queue.add (TaskOptions.Builder.url("/notifyTask").param("key",sub.getKey()));
						notified = true;
					}
					else {
						notified = Utility.notifyOneSubscriber(sub);					
					}
					if (notified) {
						count++;
					}
				}
			}
		} finally {
			query.closeAll();
		}
		return count;
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * make a pass through the database and delete subscriptions that have expired.
	 * should probably be invoked by a GAE cron job on an hourly basis.
	 * @param deltaMillis kill subscribers with a timestamp older than this
	 * @return success flag
	 */
	public static boolean expireSubscriptions (long deltaMillis) {
		//TODO Needs some logic here...
		return true;
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * calculate the MD5 hash of a given string
	 * @param arg the string to calculate the MD5 hash for
	 * @return a hex string of the MD5 hash
	 */
	public static String makeMD5 (String arg) {
		//compute the checksum
		String checksum = "";
		try {
			MessageDigest digest = java.security.MessageDigest.getInstance ("MD5");
			digest.update(arg.getBytes());
			checksum = toHexString (digest.digest());
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		}
		return checksum;
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * convert an array of bytes to a hex string
	 * @param bytes
	 * @return hex string
	 */
	public static String toHexString(byte bytes[]) {
		StringBuffer retString = new StringBuffer();
		for (int i = 0; i < bytes.length; ++i) {
			retString.append(Integer.toHexString(0x0100 + (bytes[i] & 0x00FF)).substring(1));
		}
		return retString.toString();
	}

}
